/* filename:    homepm.g
** description: The program maximizes the posterior density of the 
**              home production models
**              with respect to the model parameters
** created:     07/10/00
*/

library pgraph, user, homelib;
cls;
/* Global defaults for NLSYS procedure
*/
  __output = 1;
  _FIX1 = 0;
  _FIX2 = 0;

/******************************************************** 
**      Estimate the DSGE Model
**      Models: 1 = RBC
**              2 = HOME Production
*/ 
#include c:\projects\active\homep\Gauss\pathspec.g
lmodel = "m1";
mspec  = 2;
mprior = 1;
npara  = 18;

/******************************************************** 
** Import data on productivity growth, home investment growth,
** and hours: series (nobs,3)
** observations from 1965:I to 1997:IV
*/
#include c:\projects\active\homep\Gauss\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   /* Note that YY is defined defined over the entire series */
T0    = 20;

/******************************************************** 
** Import the information for the prior density
*/
lprior = ftos(mprior,"%*.*lf",1,0);

priorfile = lmodel $+ lprior $+ "pri.out";
load path = ^priopath prior[npara+1,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];

/******************************************************** 
** Starting Values for Maximization
*/
alp = 0.74577;
bet = 0.9775;
gam = 0.0036934;
del = 0.016245;
Lm  = 0.34055;
rhoa = 0.76495;
rhob = 0.873034;
Lh  = 0.16815;
eta = 24.53;
theta = 0.7750;
nu  = 2.1847821;
shm = 0.683938;
tau2 = 2.2659652;
psi1 = 3.1496614;
psi2 = 0.0047799;

sigz = 0.009020;
siga = 0.00838;
sigb = 0.0115;                                       


para = alp | bet | gam   | del | Lm  |rhoa | rhob |
       Lh  | eta | theta | nu  | shm | tau2 |
       psi1| psi2| sigz  | siga| sigb  ;

para_names = { ALP BET GAM DEL LM RHOA RHOB LH ETA THETA NU SHM TAU2 PSI1 PSI2 SIGZ SIGA SIGB };
save path = ^priopath para_names;

/* Load transformation scheme for parameters
*/
transfile = "trans.out";
load path=^priopath _trspec[npara,4] = ^transfile;


/******************************************************** 
**      Calculate posterior density at starting values
**
*/
lnpy = -objfcn(invtrans(para));
cls;
"Posterior at Starting Value:    " lnpy;
"Prior Density at Starting Value:" priodens(para,pmean,pstdd,pshape);
"Press key to continue";
w = keyw;
cls;

/********************************************************
**       Maximize the posterior density
**
*/
x0= invtrans(para) + 0.01*rndn(npara,1);
H0= eye(npara)*1E-4;
nit = 1000;
crit= 1E-9;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);

oparaest         = postpath $+ "\\" $+ lmodel $+ lprior $+ "mode";
create fhparaest = ^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
{lnpy, obsmean, obsvar} = evalhome( paraest,mspec,T0,YY);
lnprio  = priodens( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
" ";
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,(0~1~1~1~1~1),fmt);
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
local lnpY, lnprio, obsmean, obsvar, modelpara;

/* likelihood 
*/
modelpara = trans(para);

if lmodel $== "m2";
   modelpara[11] = 1.05;
   modelpara[13] = 1.05;
endif;

{lnpY,obsmean,obsvar} = evalhome(modelpara,mspec,T0,YY);

/* Evaluate the Prior distribution
*/
lnprio = priodens(modelpara, pmean, pstdd, pshape);

/*
locate 25,1;
"Likelihood" lnpY;
*/

retp(real(-lnpY-lnprio));  /* We minize the inverse of the likelihood fcn */
endp;
